import { Grammar } from "../grammar.js";
import { TMRUtility } from "../tmr-utility.js";
import { tmrTokenZIndex } from "../tmr-constants.js";
import { Draconique } from "./draconique.js";

export class Debordement extends Draconique {

  type() { return 'souffle' }
  match(item) { return Draconique.isSouffleDragon(item) && Grammar.toLowerCaseNoAccent(item.name).includes('debordement'); }
  manualMessage() { return false }
  async onActorCreateOwned(actor, souffle) {
    const existants = actor.items.filter(it => this.isCase(it)).map(it => it.system.coord);
    const tmr = await TMRUtility.getTMRAleatoire(it => !(TMRUtility.isCaseHumide(it) || existants.includes(it.coord)));
    await this.createCaseTmr(actor, 'Debordement', tmr, souffle.id);
  }

  code() { return 'debordement' }
  tooltip(linkData) { return `Débordement` }
  img() { return 'systems/foundryvtt-reve-de-dragon/icons/tmr/debordement.svg' }

  createSprite(pixiTMR) {
    const sprite = pixiTMR.sprite(this.code(), {
      zIndex: tmrTokenZIndex.casehumide,
      decallage: pixiTMR.sizes.decallage(0, 2/3),
      taille: () => pixiTMR.sizes.half,
    })
    return sprite;
  }

}
