import { Grammar } from "../grammar.js";
import { TMRUtility } from "../tmr-utility.js";
import { tmrTokenZIndex } from "../tmr-constants.js";
import { Draconique } from "./draconique.js";

export class FermetureCites extends Draconique {

  type() { return 'souffle' }
  match(item) { return Draconique.isSouffleDragon(item) && Grammar.toLowerCaseNoAccent(item.name).includes('fermeture des cites'); }
  manualMessage() { return false }
  async onActorCreateOwned(actor, souffle) { await this._fermerLesCites(actor, souffle); }

  code() { return 'fermeture' }
  tooltip(linkData) { return `Cité fermée` }
  img() { return 'systems/foundryvtt-reve-de-dragon/icons/tmr/fermeture.svg' }

  createSprite(pixiTMR) {
    return pixiTMR.sprite(this.code(), {
      zIndex: tmrTokenZIndex.conquete,
      decallage: pixiTMR.sizes.decallage(0, 0),
      taille: () => pixiTMR.sizes.full
    })
  }

  async _fermerLesCites(actor, souffle) {
    let existants = actor.items.filter(it => this.isCase(it)).map(it => it.system.coord);
    let ouvertes = TMRUtility.filterTMR(it => it.type == 'cite' && !existants.includes(it.coord));
    for (let tmr of ouvertes) {
      await this.createCaseTmr(actor, 'Fermeture', tmr, souffle.id);
    }
  }
}
