import { Grammar } from "../grammar.js";
import { RdDDice } from "../rdd-dice.js";
import { TMRUtility } from "../tmr-utility.js";
import { tmrTokenZIndex } from "../tmr-constants.js";
import { Draconique } from "./draconique.js";
import { TMRAnimations } from "./animation.js";

export class Periple extends Draconique {

  type() { return 'souffle' }
  match(item) { return Draconique.isSouffleDragon(item) && Grammar.toLowerCaseNoAccent(item.name).includes('periple'); }
  manualMessage() { return false }

  async onActorCreateOwned(actor, souffle) {
    let terrain = (await RdDDice.rollTotal("1d2")) == 1 ? 'sanctuaire' : 'necropole';
    let tmrs = TMRUtility.getListTMR(terrain);
    for (let tmr of tmrs) {
      await this.createCaseTmr(actor, 'Périple', tmr, souffle.id);
    }
  }


  code() { return 'periple' }
  tooltip(linkData) { return `Etape de périple` }
  img() { return 'systems/foundryvtt-reve-de-dragon/icons/tmr/periple.svg' }

  createSprite(pixiTMR) {
    return TMRAnimations.withAnimation(
      pixiTMR.sprite(this.code(), {
        zIndex: tmrTokenZIndex.conquete,
        decallage: pixiTMR.sizes.decallage(0, 0),
        taille: () => pixiTMR.sizes.half
      }),
      pixiTMR,
      TMRAnimations.changeZoom()
    )
  }
  getDifficulte(tmr) {
    switch (tmr.type) {
      case 'sanctuaire': return -3;
      case 'necropole': return -5;
    }
    return 0;
  }
}